##############################
##2013 Prefresher
##Day 6 - Data management
##Stephen Pettigrew
##Harvard University
##pettigrew@fas.harvard.edu
##############################


############################################################################
##### This R script discusses different strategies for data management in R
############################################################################


## Today we're going to use some of my favorite data: hockey data!
## Everything we're using today is publicly available on nhl.com.
## I've spent countless hours webscraping and cleaning this data to use
## on papers and in my hockey blog. We'll be using a tiny tiny subset of the
## data today to demonstrate things like subsetting data, merging datasets, 
## getting summary stats, etc.



## Let's start by setting our working directory and loading the data...

setwd("c:/users/stephen/dropbox/gov prefresher/2014 prefresher/r tutorials")
load("day6data.RData")


######################################################################
## Exploring a dataset
######################################################################

## Let's look at the data for a second.
## There's three objects, "gameinfo", "playbyplay", and "extra.playbyplay" (which we'll
## look at later):
ls()


## gameinfo is a dataset of meta data about the first 100 games of the 2013 NHL season.
## You can use the View() function to look at the data.

View(gameinfo)
colnames(gameinfo)
summary(gameinfo)


## playbyplay is some of the play-by-play data from a random sample of games early in the 2013 season

View(playbyplay)
colnames(playbyplay)
summary(playbyplay)


## Notice in the column names of playbyplay has a variable called "playtype".
## Let's use the unique() function to see what this variable actually is:

unique(playbyplay$playtype)

## unique is one of the most useful functions that I use in R. It makes R way better than
## Stata because it gives you a succinct list of all possible elements of a vector, which
## can be useful for lots of things.

## When I'm working with big datasets, I'll often make sure there aren't weird anamolies in
## the data by looking at variables using unique(). Check out the unique values of "penalty.type":

unique(playbyplay$penalty.type)

## When I was originally building this dataset, I discovered that a bunch of the values of 
## penalty types were actually player names. This told me that I made an error in my code somewhere
## earlier.









#######################################
## Subsetting Data
#######################################

## One of the most basic things you've want to do, with regard to data management,
## is subsetting data

## One of the easiest ways to do this is to use the subset() function.
## Let's use subset to get only the observations that are goals in the playbyplay data:

goals <- subset(playbyplay, playbyplay$playtype == "GOAL")

View(goals)



## Another way to accomplish the same thing is to use brackets to subset.

## Recall our discussion of dimensionality and object types in R from a couple days ago.
## The best way to avoid headaches when it comes to working with data in R is to 
## always have in your head the type of object you're working with and its dimensionality.

## playbyplay is a 2-dimensional dataframe, so if we want to subset by only the goals
## we'll need to specify that we only want the rows that a goals.

## First we can get a TRUE/FALSE vector of whether each row in the dataset is a goal

playbyplay$playtype == "GOAL"



## Now we can use that information to subset the full dataset.
## Notice the comma after "GOAL". This tells R that we only want the rows that are TRUE
## in the TRUE/FALSE vector.

playbyplay[playbyplay$playtype == "GOAL",]

## Without the comma, it gives us a "undefined columns selected" error. Get used to it and get
## good at recognizing it because it's one of the most common error messages you'll see in R:

playbyplay[playbyplay$playtype == "GOAL"]


## What if we don't want every variable in the dataset, but we just want the time that goals
## were scored? Write a line of code that gives us this vector. Hint: the time variable is
## called seconds.into.game















## Several different ways to do this, ordered by how I'd prefer to write the code:

playbyplay$seconds.into.game[playbyplay$playtype == "GOAL"] ## This is the best

playbyplay[playbyplay$playtype == "GOAL",6] ## If you happened to know that time was in column 6

playbyplay[playbyplay$playtype == "GOAL", "seconds.into.game"]

playbyplay[playbyplay$playtype == "GOAL",]$seconds.into.game ## Ugly looking code!

subset(playbyplay, playtype == "GOAL")$seconds.into.game ## Ugly ugly ugly










#######################################
## Merging and Appending Datasets together
#######################################

## One of the most common things you'll want to do in R is combine datasets together.

## I'll first teach you about merging datasets. By merge, I mean take a dataset and add
## columns to it based on data in another dataset.

## The "plyr" package has a function that will do this for you. I never use it though.
## It can be finicky, and I don't trust that it's doing what I want it to be 
## doing (probably a personality flaw of mine).

## Instead, I'm going to teach you how to merge data using the wonderful match() function.

## To merge datasets together (in R or Stata or anywhere), you're going to need an
## identifier variable that's common to both datasets. In R, these variables don't
## need to have the same name (unlike Stata).

## Let's look at what the match() function gives us using a really simple dataset:

data1 <- data.frame(id = c(1,1,2,2,2,2,4),
                    var1 = c("merging", "data", "in", "R", "is", "easy", "!"))

data2 <- data.frame(id = c(-5,-4,-3,-2,-1,0,1,2,3,4,5,6),
                    var2 = rpois(12, 6))

data1
data2

matches <- match(data1$id, data2$id)

## Match gives us a vector of numbers, which is the same length as
## the number of rows in data1.

matches
is.numeric(matches)
length(matches) == nrow(data1)
length(matches) == nrow(data2)



## This vector of numbers is the index number of data2, which correctly
## matches each observation in data1 to data2.

## So if we want to use this to actually merge the data together, 
## we can get smart about subsetting.

## What does this line of code do?

data2[matches,]

















## Answer: It gives us data2, with each row that's specified by the vector of
## index numbers in matches.

## With that, we can bind together the columns of data1 and data2[matches,]

cbind(data1, data2[matches,])

## Or we can just add in var2 to data1

data1$var2 <- data2$var2[matches]



merge(data1, data2, by = "id")




## Let's say we want to merge the gameinfo data into the playbyplay data.
## The common identifier variables in this case are playbyplay$gamenum and gameinfo$gamenumber


result <- merge(playbyplay, gameinfo, by.x = "gamenum", by.y = "gamenumber")

View(result)
dim(result)
ncol(result) == ncol(playbyplay) + ncol(gameinfo) -1 # minus one because the merge variable is only in there twice







######
## Appending datasets together.

## Sometimes you have two similar datasets that you want to append together, i.e.
## take the rows of one and stick them on the bottom of the other.

## If the datasets have the identical number of columns and they variables are
## ordered the same, you can just use rbind() to bind the rows together:

appended <- rbind(playbyplay[1:1000,], playbyplay[2001:3000,])

## But what if we want to append datasets that don't have identical columns?
## rbind() won't work:

ncol(playbyplay)
ncol(extra.playbyplay) ## this dataset is missing the "strength" and "period" variables

rbind(playbyplay, extra.playbyplay) ## rbind fail




## There's other, more tedious, ways to get datasets with unbalanced columns
## to bind together. The most straightforward way, though, is to use rbind.fill()
## which is in the "plyr" package. This function appends together any variables
## which have the same name in both datasets. For columns that are unique to one
## dataset, it appends them to a vector of NAs in the other dataset

install.packages("plyr")
require(plyr)

appended <- rbind.fill(playbyplay, extra.playbyplay)

## Write some code that confirms that the datasets are appended together...



















nrow(appended) == nrow(extra.playbyplay) + nrow(playbyplay)


## How does this line of code confirm that the append worked?
all(unique(appended$gamenum) %in% c(unique(playbyplay$gamenum), 
                                    unique(extra.playbyplay$gamenum)))







#######################################
## Collapsing data and getting summary stats
#######################################

## Another common thing that you're going to want to do when managing data
## in R is collapsing data into summary statistics of variables.

## This is easy to do in Stata, but here's two reasons it's better in R:
## 1. You can keep your original data and your collapsed data open at the same
##    time in R. Once you collapse in Stata, the original data disappears.
## 2. Unlike Stata, you aren't limited to just collapsing data to the mean, sd, 
##    etc. of variables in R. You can write your own function that does anything
##    in R, and then collapse using it.

## Let's say we want to get the mean of the "seconds.into.game" variable for each unique
## value of "gamenum" in the playbyplay data.

## Naively, we could write a loop. But it would be slow and inefficient:

for(i in unique(playbyplay$gamenum)){
  
  print(mean(playbyplay$seconds.into.game[playbyplay$gamenum == i]))
}

## Instead we can use one of a few different methods for doing the collapse:

## aggregate():

aggregate(playbyplay$seconds.into.game ~ playbyplay$gamenum,
          FUN = mean)

aggregate(playbyplay$seconds.into.game,
          by = list(playbyplay$gamenum),
          FUN = mean)


## by():

by(playbyplay$seconds.into.game, playbyplay$gamenum, mean)

# Or as a matrix:
as.matrix(by(playbyplay$seconds.into.game, as.factor(playbyplay$gamenum), mean))



## There's also other packages (including plyr) which have functions to do similar things.




## Write some code that uses either by() or aggregate() to get the total 
## number of hits of each game in playbyplay. Hint: do this by writing a 
## function that sums the number
## of observations where "playtype" is HIT







hits <- function(x){sum(x == "HIT")}

by(playbyplay$playtype, playbyplay$gamenum, FUN = hits)

aggregate(playbyplay$playtype ~ playbyplay$gamenum, FUN = hits)
aggregate(playbyplay$playtype, 
          by = list(playbyplay$gamenum),
          FUN = hits)






#######################################
## Missing Data
#######################################

## One key thing that will often give you errors in R is when you have missingness in
## your dataframe.

## If you try to take the mean of the "team" variable, it doesn't work...
playbyplay$team

mean(playbyplay$team)

## Some of the basic functions have built in ways to ignore missingness:

mean(playbyplay$team, na.rm = T)

## But lots of functions don't have this...

## The most naive way to get rid of missing data is to drop any rows of data
## that have missingness. We can do this by using the na.omit() function. But look what
## happens when we use this function on playbyplay:

holder <- na.omit(playbyplay)
nrow(holder)
dim(holder)

## It cuts out every row in the dataset! This is because for each row it looks for a single
## missing value. If there's just one, it cuts the row out of the dataset. By construction,
## every row in the playbyplay dataset has at least one NA. Thus, na.omit() won't help us much.



## Another useful function that looks for missing data is is.na(). Let's apply this function
## to the "team" variable and see what happens:

is.na(playbyplay$team)

## It gives us a vector of TRUEs and FALSEs, indicating whether each element in the vector is
## missing or not.
## We can use this vector for subset our data to only be the non-missing values and then 
## calculate the mean:

mean(playbyplay$team[!is.na(playbyplay$team)])

mean(playbyplay$team[!is.na(playbyplay$team)]) == mean(playbyplay$team, na.rm = T)

## Notice that I put an exclamation point inside my brackets. This told R to turn all the
## TRUEs into FALSEs and vice versa. If we didn't have this, we'd be telling R to subset the
## "team" vector to only include values that ARE missing.










#######################################
## Other random stuff
#######################################

#### Labeling columns or rows in a dataframe or matrix:

rownames(gameinfo)
rownames(gameinfo) <- 10:(nrow(gameinfo) + 9)
rownames(gameinfo)

colnames(gameinfo)
colnames(gameinfo)[5] <- "final.score.home"
colnames(gameinfo)



#### Ordering and sorting variables:

## Sort gives you the actual values of the variable, sorted in ascending order
sort(playbyplay$playnum)

## Order gives you the row numbers in the order that you sorted them
order(playbyplay$playnum)
View(playbyplay[order(playbyplay$playnum),])



#### Dropping variables:

colnames(extra.playbyplay)
extra.playbyplay$stop.type <- NULL
colnames(extra.playbyplay)




#### Converting factor variables into their character or numeric tags:

## Sometimes you read data into R (especially from csvs) that you think is numeric,
## but it's actually a factor. This can be a headache.

## Here's an example with 15 randomly sampled numbers between 20 and 25:

set.seed(8787)
data <- as.factor(sample(20:25, 15, replace = T))
data

## Since it's a factor variable, we can't take the mean:

mean(data)

## So we can try to convert it to numeric, but R tells us the mean isn't between 20 and 25:

mean(as.numeric(data))

## This is because as.numeric(data) converts the variable to the (arbitrary) numeric tags
## it assigned to each of the labels

## The way to deal with this is simple:

as.numeric(as.character(data))
mean(as.numeric(as.character(data)))



