##############################
##2013 Prefresher
##Exporting R results to LaTeX
##Stephen Pettigrew
##Harvard University
##pettigrew@fas.harvard.edu
##############################


##### This R script shows how to export results from R
##### into LaTeX code. This is particularly useful when
##### you have regression output that you want to put
##### into a table in LaTeX for a problem set or a paper.


###################################
## Generating fake data and models
###################################

## We'll start by generating a fake dataset and then
## running some fake linear and non-linear regressions
## on it.

set.seed(8787)
require(boot)

data <- data.frame(x1 = rnorm(1000, mean = 5, sd = 3),
                   x2 = rpois(1000, lambda = 5))
data$y1 <- 8.6 + 3.7 * data$x1 - 1.2 * data$x2 + rnorm(1000)
data$y2 <- -1.3 - .42 * data$x1 + 0.88 * data$x2 + rnorm(1000)
data$y2 <- inv.logit(data$y2)
data$y2 <- rbinom(1000, size = 1, prob = data$y2)


## Now we'll estimate a couple regressions and MLE models
## which we'll export the results of to LaTeX

# linear models using glm() and zelig():
model1 <- lm(y1 ~ x1,
             data = data)
model2 <- lm(y1 ~ x2,
             data = data)
model3 <- lm(y1 ~ x1 + x2,
             data = data)

# logit models:

model4 <- glm(y2 ~ x1,
              data = data,
              family = "binomial")

install.packages("Zelig")
require(Zelig)

model5 <- zelig(y2 ~ x2,
                data = data,
                model = "logit")

model6 <- glm(y2 ~ x1 + x2,
              data = data,
              family = "binomial")

## We can look at the results of our models by using summary()

summary(model2)
summary(model4)
summary(model6)
## etc.




##################################
## Exporting our results
##################################

## there's three (or more?) packages you can use to export stuff from 
## R to LaTeX

##############
## xtable
##############

## I'm pretty sure the tables that xtable creates can only have the
## results from one model (I'm not 100% sure on this).

install.packages("xtable")
require(xtable)

xtable(model1)

xtable(model1,
       caption = "my caption for model1",
       digits = 2) ## how many digits to round my results to


## xtable() is also really useful if you want to export an entire
## dataset or matrix into LaTeX

xtable(data)
xtable(head(data))






###############
## apsrtable
###############

## The function apsrtable() in the apsrtable package exports regression
## results in the same format as you'd see in the American Political
## Science Review

## !!!! Note that if you copy and paste the code from apsrtable()
## into LaTeX, you need to make sure you have \usepackage{dcolumn}
## in the preamble of your TeX file.

install.packages("apsrtable")
require(apsrtable)
apsrtable(model1)

## Unlike xtable() you can include the results from multiple regressions
## into one table

apsrtable(model1, model2, model3)







###############
## stargazer
###############

## The third package for exporting stuff from R to LaTeX is stargazer.
## This is the one that I use because it's the best and most
## customizable and because it was created (mostly in the CGIS computer
## lab) by Marek Hlavac, who's a PEG student in my cohort here 
## at Harvard. So you should use it too.

install.packages("stargazer")
require(stargazer)

stargazer(model4)

## Stargazer is also really nice because you can export the results
## from multiple models. It does a nice job of not only labeling
## what the dependent variable is in each model (i.e. y1 or y2 
## in our case), or the type of model (linear or logit in our models).

stargazer(model1, model4, model5, model6)


## Stargazer is a lot more customizable than xtable() or asprtable()
## and it's compatible with results that come out of lm(), glm(),
## zelig(), and a ton of others
## Basically anything that appears in the table it print out can
## be turned on or off or customized. Check out the help file if
## to find how to do these things.



## You can also use stargazer() to export summary statistics from
## a dataframe

stargazer(data)

## Or you can have it export the dataset itself (similar to xtable())
stargazer(data[1:25,], summary = F) ## export the first 25 lines of data